@@ -27,8 +27,16 @@ BrowserBridge *CreateBrowserBridge ();
 G_END_DECLS
 
 class BrowserBridge {
+	Surface *surface;
+	bool shutting_down;
+
  public:
+	BrowserBridge () : surface(NULL), shutting_down(0) {}
 	virtual DownloaderRequest* CreateDownloaderRequest (const char *method, const char *uri, bool disable_cache) = 0;
+	virtual void Shutdown () { shutting_down = true; }
+	bool IsShuttingDown () { return shutting_down; }
+	void SetSurface (Surface *value) { surface = value; }
+	Surface *GetSurface () { return surface; }
 };
 
 #endif /* BROWSER_BRIDGE */
